﻿using Hotline.Article;
using Hotline.FlowEngine.Workflows;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Dtos;
using Hotline.Users;
using Microsoft.AspNetCore.Mvc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XF.Domain.Authentications;
using XF.Domain.Dependency;
using XF.Domain.Repository;
using SqlSugar;

namespace Hotline.Orders
{
    public class ComplementOrderService : IComplementOrderService, IScopeDependency
    {
        private readonly ISessionContext _sessionContext;
        private readonly IRepository<OrderComplement> _orderComplementRepository;
        private readonly IRepository<WorkflowStep> _workflowStepRepository;
        private readonly IRepository<User> _userRepository;
        private readonly ICircularRecordDomainService _circularRecordDomainService;
        private readonly IRepository<OrderComplementCopy> _orderComplementCopyRepository;

        public ComplementOrderService(ISessionContext sessionContext,
           IRepository<OrderComplement> orderComplementRepository,
           IRepository<WorkflowStep> workflowStepRepository,
           IRepository<User> userRepository,
           ICircularRecordDomainService circularRecordDomainService,
           IRepository<OrderComplementCopy> orderComplementCopyRepository)
        {
            _sessionContext = sessionContext;
            _orderComplementRepository = orderComplementRepository;
            _workflowStepRepository = workflowStepRepository;
            _userRepository = userRepository;
            _circularRecordDomainService = circularRecordDomainService;
            _orderComplementCopyRepository = orderComplementCopyRepository;
        }

        /// <summary>
        /// 补充列表查询
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public ISugarQueryable<OrderComplement> GetOrderComplementList(ComplementOrderRequestDto dto)
        {
            return _orderComplementRepository.Queryable()
                  .Includes(p => p.Order)
                  .WhereIF(_sessionContext.OrgIsCenter == false, p => p.CreatorOrgId.StartsWith(_sessionContext.RequiredOrgId))
                  .WhereIF(!string.IsNullOrEmpty(dto.No), p => p.Order.No == dto.No)
                  .WhereIF(!string.IsNullOrEmpty(dto.Title), p => p.Order.Title == dto.Title)
                  .WhereIF(!string.IsNullOrEmpty(dto.CurrentStepName), p => p.Order.CurrentStepName == dto.CurrentStepName)
                  .WhereIF(!string.IsNullOrEmpty(dto.SupplyName), p => p.SupplyName == dto.SupplyName)
                  .WhereIF(dto.StartTime.HasValue, p => p.SupplyTime >= dto.StartTime)
                  .WhereIF(dto.EndTime.HasValue, p => p.SupplyTime <= dto.EndTime)
                  .OrderByDescending(p => p.SupplyTime);

        }

        /// <summary>
        /// 补充操作记录列表查询
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public ISugarQueryable<OrderComplementCopy> GetOrderComplementCopyList(ComplementOrderCopyRequestDto dto)
        {
            return _orderComplementCopyRepository.Queryable()
                  .Includes(p => p.Order)
                  .WhereIF(_sessionContext.OrgIsCenter == false, p => p.CreatorOrgId.StartsWith(_sessionContext.RequiredOrgId))
                  .WhereIF(!string.IsNullOrEmpty(dto.No), p => p.Order.No == dto.No)
                  .WhereIF(!string.IsNullOrEmpty(dto.Title), p => p.Order.Title == dto.Title)
                  .WhereIF(!string.IsNullOrEmpty(dto.CurrentStepName), p => p.Order.CurrentStepName == dto.CurrentStepName)
                  .WhereIF(!string.IsNullOrEmpty(dto.SupplyName), p => p.CreatorName == dto.SupplyName)
                  .WhereIF(dto.StartTime.HasValue, p => p.CreationTime >= dto.StartTime)
                  .WhereIF(dto.EndTime.HasValue, p => p.CreationTime <= dto.EndTime)
                  .WhereIF(!string.IsNullOrEmpty(dto.OperationType), p => p.OperationType == dto.OperationType)
                  .OrderByDescending(p => p.CreationTime);

        }
    }
}
